@@ -2,18 +2,18 @@ use node_workers::{EmptyPayload, WorkerPool};
 
 fn main() {
   {
-    // Create a pool of 2 node workers
     let mut pool = WorkerPool::setup(2);
     pool.with_debug(true);
-    // Create 4 payloads
     let payloads = vec![10, 20, 30, 40];
     let result = pool
-      .run_task::<u64, _>("examples/worker", "fib2", payloads)
+      .perform::<u64, _>("examples/worker", "fib2", payloads)
       .unwrap();
     println!("-----");
     println!("result: {:?}", result);
   }
+
   {
+    // Using serde, results from workers can be deserialized into structs
     use serde::{Deserialize, Serialize};
     #[derive(Serialize, Deserialize, Debug)]
     struct Person {
@@ -21,12 +21,10 @@ fn main() {
       age: u8,
       phones: Vec<String>,
     }
-    // Create a pool of 2 node workers
+
     let mut pool = WorkerPool::setup(2);
-    // Create 4 payloads
-    let payloads = vec![EmptyPayload::new()];
     let result = pool
-      .run_task::<Person, _>("examples/worker", "getUser", payloads)
+      .perform::<Person, _>("examples/worker", "getUser", vec![EmptyPayload::new()])
       .unwrap();
     println!("-----");
     println!("result: {:?}", result);
