@@ -109,6 +109,24 @@ impl Worker {
     Ok(result_str)
   }
 
+  pub fn wait_for_ready(&mut self) -> Result<()> {
+    if !self.ready {
+      let mut reader = self.stdout.take().unwrap();
+      let stdin = self.stdin.take().unwrap();
+      let mut child = self.child.take().unwrap();
+
+      self
+        .communicate("", "READY", &stdin, &mut reader, &mut child)
+        .context("communicating with process")?;
+      self.ready = true;
+
+      self.stdout = Some(reader);
+      self.stdin = Some(stdin);
+      self.child = Some(child);
+    }
+    Ok(())
+  }
+
   pub fn communicate(
     &self,
     send: &str,
