@@ -1,18 +1,19 @@
-use crate::{as_payload::AsPayload, print_debug, worker::Worker, worker_thread::WorkerThread};
+use crate::{
+  as_payload::AsPayload, print_debug, worker_pool_inner::WorkerPoolInner,
+  worker_thread::WorkerThread,
+};
 use anyhow::{bail, Result};
 use serde::de::DeserializeOwned;
-use std::sync::{
-  atomic::{AtomicUsize, Ordering},
-  Arc, Mutex,
+use std::{
+  sync::{Arc, Mutex},
+  thread::JoinHandle,
 };
 
-/// A pool of nodejs workers
+/// A pool of nodejs workers.
+/// Wraps a inner struct inside `Arc<Mutex<T>>` to be able to invoke it's method within a spawned thread.
+/// This is important so that indefinitely blocking methods such as `get_available_workers` can be offloaded.
 pub struct WorkerPool {
-  binary_args: Vec<String>,
-  workers: Vec<Arc<Mutex<Worker>>>,
-  max_workers: usize,
-  busy_counter: Arc<AtomicUsize>,
-  debug: bool,
+  inner: Arc<Mutex<WorkerPoolInner>>,
 }
 
 impl WorkerPool {
@@ -25,11 +26,7 @@ impl WorkerPool {
   /// ```
   pub fn setup(max_workers: usize) -> Self {
     WorkerPool {
-      binary_args: vec!["node".into()],
-      workers: Vec::new(),
-      max_workers,
-      busy_counter: Arc::new(AtomicUsize::new(0)),
-      debug: false,
+      inner: Arc::new(Mutex::new(WorkerPoolInner::setup(max_workers))),
     }
   }
 
@@ -37,20 +34,22 @@ impl WorkerPool {
   /// This can be usefull configure node or to run JS via another runtime
   /// ```rust
   /// use node_workers::{EmptyPayload, WorkerPool};
+  /// # use std::error::Error;
   ///
+  /// # fn main() -> Result<(), Box<dyn Error>> {
   /// let mut pool = WorkerPool::setup(4);
   /// pool.set_binary("node -r esbuild-register");
-  /// pool
-  ///   .perform::<(), _>("examples/worker.ts", "ping", EmptyPayload::bulk(1))
-  ///   .unwrap();
+  /// pool.perform::<(), _>("examples/worker.ts", "ping", EmptyPayload::bulk(1))?;
+  /// # Ok(())
+  /// # }
   /// ```
   pub fn set_binary(&mut self, binary: &str) {
-    self.binary_args = shell_words::split(binary).expect("couldn't parse binary");
+    self.inner.lock().unwrap().set_binary(binary);
   }
 
   /// Enable or disable logging
   pub fn with_debug(&mut self, debug: bool) {
-    self.debug = debug;
+    self.inner.lock().unwrap().with_debug(debug);
   }
 
   /// Run a single worker in a thread. This method returns the created thread, not the result of the worker.
@@ -66,18 +65,18 @@ impl WorkerPool {
   /// ```
   ///
   /// The returned thread optionally holds the serialized result from the worker. This can be deserialized using serde_json in order to
-  /// get a proper result.
+  /// get a proper result. This is done under the hood for you.
   /// ```
   /// use node_workers::{WorkerPool};
+  /// # use std::error::Error;
   ///
+  /// # fn main() -> Result<(), Box<dyn Error>> {
   /// let mut pool = WorkerPool::setup(2);
-  /// let handle = pool.run_worker("examples/worker", "fib2", 40u32);
-  /// let result = handle
-  ///   .join()
-  ///   .unwrap()
-  ///   .map(|x| serde_json::from_str::<u32>(x.as_str()).unwrap())
-  ///   .unwrap();
-  /// println!("run_worker result: {}", result);
+  /// let thread = pool.run_worker("examples/worker", "fib2", 40u32);
+  /// let result = thread.get_result::<u32>()?;
+  /// println!("run_worker result: {:#?}", result);
+  /// # Ok(())
+  /// # }
   /// ```
   pub fn run_worker<P: AsPayload>(
     &mut self,
@@ -85,39 +84,18 @@ impl WorkerPool {
     cmd: &str,
     payload: P,
   ) -> WorkerThread {
-    let worker = self.get_available_worker();
-    self.busy_counter.fetch_add(1, Ordering::SeqCst);
-    print_debug!(
-      self.debug,
-      "[pool] got worker {}",
-      worker.lock().unwrap().id
-    );
-    let file_path = String::from(file_path);
-    let waiting = self.busy_counter.clone();
-    let cmd = cmd.to_string();
-    let debug = self.debug;
-    let binary_args = self.binary_args.clone();
     let payload = payload.to_payload();
+    let file_path = file_path.to_string();
+    let cmd = cmd.to_string();
+    let inner = self.inner.clone();
 
+    // spawn a thread so that inner.get_available_worker() doesn't block
     let handle = std::thread::spawn(move || {
-      let worker = worker.clone();
-      worker
-        .lock()
-        .unwrap()
-        .init(binary_args, file_path.as_str())
-        .unwrap();
-      let res = worker
-        .lock()
-        .unwrap()
-        .perform_task(cmd, payload)
-        .expect("perform task");
-      print_debug!(
-        debug,
-        "[pool] performed task on worker {}",
-        worker.lock().unwrap().id
-      );
-      waiting.fetch_sub(1, Ordering::SeqCst);
-      res
+      let inner = inner.clone();
+      let mut pool = inner.lock().unwrap();
+      let res = pool.run_worker(file_path, cmd, payload);
+      drop(pool);
+      res.join().unwrap()
     });
     WorkerThread::from_handle(handle)
   }
@@ -127,33 +105,46 @@ impl WorkerPool {
   /// Contrarily to `run_worker`, this method is blocking and directly return the result from all workers.
   /// ```
   /// use node_workers::{WorkerPool};
+  /// # use std::error::Error;
   ///
+  /// # fn main() -> Result<(), Box<dyn Error>> {
   /// let mut pool = WorkerPool::setup(2);
   /// pool.with_debug(true);
   /// let payloads = vec![10, 20, 30, 40];
   /// let result = pool.perform::<u64, _>("examples/worker", "fib2", payloads).unwrap();
-  /// println!("result: {:?}", result);
+  /// println!("result: {:#?}", result);
+  /// # Ok(())
+  /// # }
   /// ```
+  /// ## Errors
+  ///
+  /// Each worker is run in a thread, and `perform()` will return an error variant if one of them panick.
   pub fn perform<T: DeserializeOwned, P: AsPayload>(
     &mut self,
     file_path: &str,
     cmd: &str,
     payloads: Vec<P>,
   ) -> Result<Vec<Option<T>>> {
-    print_debug!(self.debug, "[pool] running tasks");
+    let debug = self.inner.lock().unwrap().debug;
+    print_debug!(debug, "[pool] running tasks");
     let mut handles = Vec::new();
     for (n, payload) in payloads.into_iter().map(|x| x.to_payload()).enumerate() {
-      print_debug!(self.debug, "[pool] (task {}) start of iteration", n);
-      let handle = self.run_worker(file_path, cmd, payload);
+      print_debug!(debug, "[pool] (task {}) start of iteration", n);
+      let handle =
+        self
+          .inner
+          .lock()
+          .unwrap()
+          .run_worker(file_path.to_string(), cmd.to_string(), payload);
       handles.push(handle);
-      print_debug!(self.debug, "[pool] (task {}) end of iteration", n);
+      print_debug!(debug, "[pool] (task {}) end of iteration", n);
     }
 
     handles
       .into_iter()
       .enumerate()
       .map(|(n, x)| {
-        print_debug!(self.debug, "[pool] (thread {}) joined", n);
+        print_debug!(debug, "[pool] (thread {}) joined", n);
         let res = x.get_result::<T>();
         if let Ok(res) = res {
           Ok(res)
@@ -164,33 +155,28 @@ impl WorkerPool {
       .collect::<Result<Vec<_>, _>>()
   }
 
-  fn get_available_worker(&mut self) -> Arc<Mutex<Worker>> {
-    let idle_worker = self.workers.iter().find(|w| {
-      if let Ok(w) = w.try_lock() {
-        return w.idle;
-      }
-      false
-    });
-    if let Some(idle_worker) = idle_worker {
-      idle_worker.lock().unwrap().idle = false;
-      print_debug!(self.debug, "[pool] found idle worker");
-      return idle_worker.clone();
-    }
-    if self.workers.len() < self.max_workers {
-      let mut worker = Worker::new(self.workers.len() + 1, self.debug);
-      worker.idle = false;
-      self.workers.push(Arc::new(Mutex::new(worker)));
-      print_debug!(self.debug, "[pool] created new worker");
-      return self.workers.last().unwrap().clone();
-    }
-    print_debug!(self.debug, "[pool] waiting for worker to be free");
-    loop {
-      if self.busy_counter.load(Ordering::SeqCst) == 0 {
-        print_debug!(self.debug, "[pool] pool is free");
-        break;
-      }
-    }
-    self.get_available_worker()
+  /// Boot a maximum of *n* workers, making them ready to take on a task right away.
+  /// ```rust
+  /// use node_workers::{WorkerPool};
+  ///
+  /// let mut pool = WorkerPool::setup(2);
+  /// let handle = pool.warmup(2, "examples/worker");
+  ///
+  /// //... some intensive task on the main thread
+  ///
+  /// handle.join().expect("Couldn't warmup workers");
+  /// //... task workers
+  /// ```
+  pub fn warmup(&self, nbr_workers: usize, file_path: &str) -> JoinHandle<()> {
+    let inner = self.inner.clone();
+    let file_path = file_path.to_string();
+    std::thread::spawn(move || {
+      inner
+        .lock()
+        .unwrap()
+        .warmup(nbr_workers, &file_path)
+        .unwrap()
+    })
   }
 }
 
@@ -200,46 +186,89 @@ mod tests {
 
   #[test]
   pub fn create_worker_when_needed() {
-    let mut pool = WorkerPool::setup(1);
-    assert_eq!(pool.workers.len(), 0);
+    let pool = WorkerPool::setup(1);
+    assert_eq!(pool.inner.lock().unwrap().workers.len(), 0);
 
-    pool.get_available_worker();
-    assert_eq!(pool.workers.len(), 1);
+    pool.inner.lock().unwrap().get_available_worker();
+    assert_eq!(pool.inner.lock().unwrap().workers.len(), 1);
   }
 
   #[test]
   pub fn same_idle_worker() {
-    let mut pool = WorkerPool::setup(1);
-    let worker = pool.get_available_worker();
+    let pool = WorkerPool::setup(1);
+    let worker = pool.inner.lock().unwrap().get_available_worker();
     worker.lock().unwrap().idle = true;
     let worker_id = worker.lock().unwrap().id;
-    let other_worker_id = pool.get_available_worker().lock().unwrap().id;
+    let other_worker_id = pool
+      .inner
+      .lock()
+      .unwrap()
+      .get_available_worker()
+      .lock()
+      .unwrap()
+      .id;
     assert_eq!(worker_id, other_worker_id);
   }
 
   #[test]
   pub fn create_new_worker_when_busy() {
-    let mut pool = WorkerPool::setup(2);
-    pool.run_worker("examples/worker", "fib2", 40);
+    let pool = WorkerPool::setup(2);
+    pool
+      .inner
+      .lock()
+      .unwrap()
+      .run_worker("examples/worker".into(), "fib2".into(), 40);
 
-    let worker_id = pool.get_available_worker().lock().unwrap().id;
+    let worker_id = pool
+      .inner
+      .lock()
+      .unwrap()
+      .get_available_worker()
+      .lock()
+      .unwrap()
+      .id;
+    println!("got worker_id");
     assert_eq!(worker_id, 2);
   }
 
   #[test]
   pub fn reuse_worker_when_full() {
-    let mut pool = WorkerPool::setup(1);
-    pool.run_worker("examples/worker", "fib2", 40);
+    let pool = WorkerPool::setup(1);
+    pool
+      .inner
+      .lock()
+      .unwrap()
+      .run_worker("examples/worker".into(), "fib2".into(), 40);
 
-    let worker_id = pool.get_available_worker().lock().unwrap().id;
+    let worker_id = pool
+      .inner
+      .lock()
+      .unwrap()
+      .get_available_worker()
+      .lock()
+      .unwrap()
+      .id;
     assert_eq!(worker_id, 1);
   }
 
+  #[test]
+  pub fn warmup() {
+    let mut pool = WorkerPool::setup(2);
+    pool.with_debug(true);
+    pool.warmup(2, "examples/worker").join().unwrap();
+
+    let workers = pool.inner.lock().unwrap().workers.clone();
+    for worker in workers {
+      assert_eq!(worker.lock().unwrap().ready, true);
+    }
+  }
+
   #[test]
   pub fn error_invalid_command() {
     {
       let mut pool = WorkerPool::setup(1);
       let res = pool.run_worker("foo", "fib2", 40).join();
+      println!("{:?}", res);
       assert_eq!(true, matches!(res, Err(_)));
     }
 
@@ -248,6 +277,12 @@ mod tests {
       let res = pool.perform::<(), _>("foo", "fib2", vec![40]);
       assert_eq!(true, matches!(res, Err(_)));
     }
+
+    {
+      let pool = WorkerPool::setup(1);
+      let res = pool.warmup(1, "foo").join();
+      assert_eq!(true, matches!(res, Err(_)));
+    }
   }
 
   #[test]
